@@ -1,9 +1,10 @@
 from kraken.base_api.base_api import KrakenBaseRestAPI
+from typing import List
+
 
 class TradeClient(KrakenBaseRestAPI):
 
-    def create_order(
-        self,
+    def create_order(self,
         ordertype: str,
         side: str,
         volume: str,
@@ -13,7 +14,7 @@ class TradeClient(KrakenBaseRestAPI):
         trigger: str=None,
         leverage: str=None,
         stp_type: str='cancel-newest',
-        oflags: [str]=None,
+        oflags: List[str]=None,
         timeinforce: str=None,
         starttm: str='0',
         expiretm: str='0',
@@ -48,10 +49,10 @@ class TradeClient(KrakenBaseRestAPI):
         if close_ordertype != None: params['close[ordertype]'] = close_ordertype
         if close_price != None: params['close[price]'] = close_price
         if close_price2 != None: params['close[price2]'] = close_price2
-        return self._request('POST', '/private/AddOrder', params=params)
+        return self._request(method='POST', uri='/private/AddOrder', params=params)
 
 
-    def create_order_batch(self, orders: [dict], pair: str, deadline: str=None, validate: bool=False) -> dict:
+    def create_order_batch(self, orders: List[dict], pair: str, deadline: str=None, validate: bool=False) -> dict:
         '''https://docs.kraken.com/rest/#operation/addOrderBatch'''
         params = {
             'orders': orders,
@@ -59,7 +60,7 @@ class TradeClient(KrakenBaseRestAPI):
             'validate': validate
         }
         if deadline != None: params['deadline'] = deadline
-        return self._request('POST', '/private/AddOrderBatch', params=params, do_json=True)
+        return self._request(method='POST', uri='/private/AddOrderBatch', params=params, do_json=True)
 
     def edit_order(self, 
         txid: str, 
@@ -85,25 +86,23 @@ class TradeClient(KrakenBaseRestAPI):
         if price2 != None: params['price2'] = price2
         if oflags != None: params['oflags'] = self._to_str_list(oflags)
         if cancel_response != None: params['cancel_response'] = cancel_response
-        return self._response('POST', '/private/EditOrder', params=params)
+        if deadline != None: params['deadline'] = deadline
+        return self._response('POST', uri='/private/EditOrder', params=params)
 
     def cancel_order(self, txid) -> dict:
         '''https://docs.kraken.com/rest/#operation/cancelOrder'''
-        params = { 'txid': txid }
-        return self._request('POST', '/private/CancelOrder', params=params)
+        return self._request(method='POST', uri='/private/CancelOrder', params={ 'txid': txid })
 
     def cancel_all_orders(self) -> dict:
         '''https://docs.kraken.com/rest/#operation/cancelAllOrders'''
-        return self._request('POST', '/private/CancelAll')
+        return self._request(method='POST', uri='/private/CancelAll')
 
     def cancel_all_orders_after_x(self, timeout: int) -> dict:
         '''https://docs.kraken.com/rest/#operation/cancelAllOrdersAfter'''
-        params = { 'timeout': timeout }
-        return self._request('POST', '/private/CancelAllOrdersAfter', params=params)
+        return self._request(method='POST', uri='/private/CancelAllOrdersAfter', params={ 'timeout': timeout })
 
-    def cancel_order_batch(self, orders: [str]) -> dict:
+    def cancel_order_batch(self, orders: List[str]) -> dict:
         '''https://docs.kraken.com/rest/#operation/cancelOrderBatch'''
-        params = { 'orders': orders }
-        return self._request('POST', '/private/CancelOrderBatch', params=params)
+        return self._request(method='POST', uri='/private/CancelOrderBatch', params={ 'orders': orders })
 
 
